﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using CommunityToolkit.Mvvm.ComponentModel;
using CtrEditor.FuncionesBase;
using DocumentFormat.OpenXml.Spreadsheet;
using Newtonsoft.Json;
using Xceed.Wpf.Toolkit.PropertyGrid.Attributes;
using Color = System.Windows.Media.Color;
using Colors = System.Windows.Media.Colors;
using JsonIgnoreAttribute = Newtonsoft.Json.JsonIgnoreAttribute;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucFramePlate.xaml
    /// </summary>
    /// 

    public partial class osFramePlate : osBase, IosBase
    {
        [JsonIgnore]
        public float offsetY;
        [JsonIgnore]
        public float offsetX;

        public static string NombreClase()
        {
            return "Frame Plate";
        }
        private string nombre = NombreClase();
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        [property: Description("Layer index to add to Objects on the Frame")]
        [property: Category("Layer:")]
        int zindex_FramePlate;

        partial void OnZindex_FramePlateChanged(int value)
        {
            UpdateZIndex(value);
        }

        [ObservableProperty]
        Color color;
        [ObservableProperty]
        Color color_Titulo;

        [ObservableProperty]
        string titulo;
        [ObservableProperty]
        public float alto_Titulo;

        // Encoder X
        [ObservableProperty]
        [property: Description("This is a link to a Encoder for X.")]
        [property: Category("Encoders:")]       
        [property: ItemsSource(typeof(osBaseItemsSource<osEncoderMotorLineal>))]
        private string encoder_X;

        [ObservableProperty]
        [property: Description("K Pulses per meter for Moving")]
        [property: Category("Encoders:")]
        public float k_encoder_X;

        [ObservableProperty]
        [property: ReadOnly(true)]
        [property: Description("Actual Position X")]
        [property: Category("Encoders:")]
        public float encoder_X_Position;


        [ObservableProperty]
        [property: Description("X in meter offset Left. Position when the encoder is 0")]
        [property: Category("Encoders:")]
        public float offset_encoder_X;

        // Encoder Y
        [ObservableProperty]
        [property: Description("This is a link to a Encoder for Y.")]
        [property: Category("Encoders:")]       
        [property: ItemsSource(typeof(osBaseItemsSource<osEncoderMotorLineal>))]
        private string encoder_Y;

        [ObservableProperty]
        [property: Description("K Pulses per meter for Moving")]
        [property: Category("Encoders:")]
        public float k_encoder_Y;

        [ObservableProperty]
        [property: ReadOnly(true)]
        [property: Description("Actual Position Y")]
        [property: Category("Encoders:")]
        public float encoder_Y_Position;

        [ObservableProperty]
        [property: Description("Y in meter offset Top. Position when the encoder is 0")]
        [property: Category("Encoders:")]
        public float offset_encoder_Y;

        partial void OnK_encoder_YChanged(float value)
        {
            UpdatePosition();
        }

        partial void OnOffset_encoder_YChanged(float value)
        {
            UpdatePosition();
        }

        partial void OnK_encoder_XChanged(float value)
        {
            UpdatePosition();
        }

        partial void OnOffset_encoder_XChanged(float value)
        {
            UpdatePosition();
        }

        [JsonIgnore]
        private osEncoderMotorLineal EncoderX;

        [JsonIgnore]
        private osEncoderMotorLineal EncoderY;

        [JsonIgnore]
        private bool isUpdatingFromEncoderX = false;

        [JsonIgnore]
        private bool isUpdatingFromEncoderY = false;

        partial void OnEncoder_XChanged(string value)
        {
            if (EncoderX != null)
                EncoderX.PropertyChanged -= OnEncoderXPropertyChanged;
            if (_mainViewModel != null && value != null && value.Length > 0)
            {
                EncoderX = (osEncoderMotorLineal)_mainViewModel.ObjetosSimulables.FirstOrDefault(s => (s is osEncoderMotorLineal && s.Nombre == value), null);
                if (EncoderX != null)
                    EncoderX.PropertyChanged += OnEncoderXPropertyChanged;
            }
        }

        partial void OnEncoder_YChanged(string value)
        {
            if (EncoderY != null)
                EncoderY.PropertyChanged -= OnEncoderYPropertyChanged;
            if (_mainViewModel != null && value != null && value.Length > 0)
            {
                EncoderY = (osEncoderMotorLineal)_mainViewModel.ObjetosSimulables.FirstOrDefault(s => (s is osEncoderMotorLineal && s.Nombre == value), null);
                if (EncoderY != null)
                    EncoderY.PropertyChanged += OnEncoderYPropertyChanged;
            }
        }

        private void OnEncoderXPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (!isUpdatingFromEncoderX)
            {
                isUpdatingFromEncoderX = true;
                // Actualizamos el nombre si este fue modificado
                if (e.PropertyName == nameof(osEncoderMotorLineal.Nombre))
                    Encoder_X = ((osEncoderMotorLineal)sender).Nombre;

                if (e.PropertyName == nameof(osEncoderMotorLineal.Valor_Actual))
                {
                    UpdatePosition();
                }

                isUpdatingFromEncoderX = false;
            }
        }

        private void OnEncoderYPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (!isUpdatingFromEncoderY)
            {
                isUpdatingFromEncoderY = true;
                // Actualizamos el nombre si este fue modificado
                if (e.PropertyName == nameof(osEncoderMotorLineal.Nombre))
                    Encoder_Y = ((osEncoderMotorLineal)sender).Nombre;

                if (e.PropertyName == nameof(osEncoderMotorLineal.Valor_Actual))
                {
                    UpdatePosition();
                }

                isUpdatingFromEncoderY = false;
            }
        }

        public void UpdatePosition()
        {
            // Update X position if encoder is available
            if (EncoderX != null && K_encoder_X != 0)
            {
                Encoder_Y_Position = EncoderY.Valor_Actual;
                Left = (Encoder_X_Position / k_encoder_X) + offset_encoder_X;
            }
            // Update Y position if encoder is available
            if (EncoderY != null && K_encoder_Y != 0)
            {
                Encoder_Y_Position = EncoderY.Valor_Actual;
                Top = (Encoder_Y_Position / k_encoder_Y) + offset_encoder_Y;
            }
        }

        public override void TopChanging(float oldValue, float newValue)
        {
            offsetY = newValue - oldValue;
        }
        public override void LeftChanging(float oldValue, float newValue)
        {
            offsetX = newValue - oldValue;
        }

        [ObservableProperty]
        [property: Description("Show/Hide the plate background")]
        [property: Category("Appearance:")]
        private bool showPlate;

        public osFramePlate()
        {
            Ancho = 0.5f;
            Alto = 0.5f;
            Alto_Titulo = 0.2f;
            Color = Colors.WhiteSmoke;
            Titulo = "Frame";
            Zindex_FramePlate = 0;
            ShowPlate = true;  // Default value
        }

        public override void ucLoaded()
        {
            base.ucLoaded();
            // El UserControl se ha cargado
            OnEncoder_XChanged(Encoder_X);
            OnEncoder_YChanged(Encoder_Y);
            UpdateZIndex(Zindex_FramePlate);
        }

        public override void ucUnLoaded()
        {
            base.ucUnLoaded();
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
        }

    }

    public partial class ucFramePlate : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }
        public int zIndex_fromFrames { get; set; }

        public ucFramePlate()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Highlight(bool State) { }
        public ZIndexEnum ZIndex_Base()
        {
            return ZIndexEnum.Decorativos;
        }

    }
}



